@@ -1,8 +1,10 @@
+import socket
+from datetime import datetime, date
 from networktools.path import home_path
 import asyncio
 import functools
 from asyncio import shield, wait_for
-
+from pathlib import Path
 # NetworkTools methods and classes:
 from data_rdb import Rethink_DBS
 from data_geo import GeoJSONData
@@ -17,10 +19,10 @@ from networktools.time import get_datetime_di
 from networktools.library import geojson2json
 
 # log module
-from basic_logtools.filelog import LogFile as FileLog
+from basic_logtools.filelog import LogFile
 
 # TaskTools for concurrency loop
-from tasktools.taskloop import coromask, renew, simple_fargs_out
+from tasktools.taskloop import TaskLoop
 from tasktools.scheduler import TaskScheduler
 
 # Async Socket
@@ -29,9 +31,8 @@ from tasktools.scheduler import TaskScheduler
 
 # Maths
 import time
-import ujson as json
 from multiprocessing import Lock
-
+import json
 
 # RethinkDB module
 from rethinkdb import RethinkDB
@@ -39,7 +40,18 @@ rdb = RethinkDB()
 rdb.set_loop_type('asyncio')
 
 
-def qjson(x): return json.loads(json.dumps(x))
+def default(o):
+    if isinstance(o, (date, datetime)):
+        return o.isoformat()
+
+
+def qjson(x):
+    return json.loads(
+        json.dumps(x,
+                   sort_keys=True,
+                   indent=1,
+                   default=default)
+    )
 
 
 class DragonGather(TaskScheduler):
@@ -66,7 +78,7 @@ class DragonGather(TaskScheduler):
         self.common = kwargs.get("common")
         self.rethinkdb_address = kwargs.get('rdb_address')
         self.rethinkdb_enu_address = kwargs.get('rdb_enu_address')
-        self.rethinkdb_dbname = kwargs.get('rdb_dbname')
+        self.rethinkdb_dbname = kwargs.get('rdb_dbname', 'test')
         self.sta_init = kwargs.get('status_tasks')
         self.status_tasks = kwargs.get('status_tasks')
         self.assigned_tasks = kwargs.get('assigned_tasks')
@@ -107,7 +119,7 @@ class DragonGather(TaskScheduler):
             'run_task': self.gather_data,
         }
         self.set_new_run_task(**coros_callback_dict)
-        self.log_path = home_path(kwargs.get('log_path', '~/log'))
+        self.log_path = Path(home_path(kwargs.get('log_path', '~/log')))
 
     @property
     def class_name(self):
@@ -141,33 +153,15 @@ class DragonGather(TaskScheduler):
             args = []
             # Create instances
 
-            task_client = loop.create_task(
-                coromask(
-                    self.check_client_status,
-                    args, {},
-                    simple_fargs_out)
-            )
-            task_client.add_done_callback(
-                functools.partial(
-                    renew,
-                    task_client,
-                    self.check_client_status,
-                    simple_fargs_out)
-            )
+            task_client = TaskLoop(self.check_client_status, args, {},
+                                   **{"name": "check_client_status"})
+
+            task = TaskLoop(self.check_status, args, {},
+                            **{"name": "check_status"})
+
+            task_client.create()
+            task.create()
 
-            task = loop.create_task(
-                coromask(
-                    self.check_status,
-                    args, {},
-                    simple_fargs_out)
-            )
-            task.add_done_callback(
-                functools.partial(
-                    renew,
-                    task,
-                    self.check_status,
-                    simple_fargs_out)
-            )
             if not loop.is_running():
                 loop.run_forever()
         except Exception as ex:
@@ -363,12 +357,8 @@ class DragonGather(TaskScheduler):
                 bprint(f"{k} : {s}")
 
     def load_dbdata(self, msg):
-        bprint("="*10)
-        print(len(msg), type(msg))
-        rprint("="*10)
         for k, v in msg.items():
             self.db_data[k] = v
-            bprint(f"{k} : {v}")
 
     def msg_load_stations(self):
         get_lst = {'user': 'admin',
@@ -424,13 +414,8 @@ class DragonGather(TaskScheduler):
     def load_data_task(self, loop):
         # bprint("Load data task")
         args = []
-        task = loop.create_task(coromask(self.load_data,
-                                         args, {},
-                                         simple_fargs_out))
-        task.add_done_callback(functools.partial(renew,
-                                                 task,
-                                                 self.load_data,
-                                                 simple_fargs_out))
+        task = TaskLoop(self.load_data, args, {}, **{"name": "load_data_task"})
+        task.create()
 
     def add_sta_instance_origin(self, ids, loop):
         # create bridge instance
@@ -442,7 +427,8 @@ class DragonGather(TaskScheduler):
         code_db = self.stations[ids]['db']
         # idd = self.get_id_by_code('DBDATA', code_db)
         # bprint("The db_data's id: %s" % idd)
-        print("Conexión source", self.rethinkdb_address)
+        print("Conexión source", self.rethinkdb_address,
+              f"DBNAME -> {self.rethinkdb_dbname}")
         db_datos = dict(host='localhost',
                         name="Source_%s" % code_db,
                         code=code_db,
@@ -450,7 +436,7 @@ class DragonGather(TaskScheduler):
                         address=self.rethinkdb_address,
                         dbname=self.rethinkdb_dbname,
                         io_loop=loop,
-                        log_path=self.log_path/"source_rdb",
+                        log_path=str(self.log_path/"source_rdb"),
                         env='gather_%s' % code)
         name = 'RethinkDB'
         self.sta_init[ids] = True
@@ -483,7 +469,7 @@ class DragonGather(TaskScheduler):
                             address=self.rethinkdb_enu_address,
                             dbname='enu_data',
                             io_loop=loop,
-                            log_path=self.log_path/"enu_rdb",
+                            log_path=str(self.log_path/"enu_rdb"),
                             env='enu_%s' % code)
 
         name = 'RethinkDB'
@@ -513,7 +499,7 @@ class DragonGather(TaskScheduler):
         kwargs['code'] = CODE
         kwargs['station'] = self.stations[ids]
         kwargs['position'] = self.position[ids]
-        kwargs['log_path'] = self.log_path/'geo_json_data'
+        kwargs['log_path'] = str(self.log_path/'geo_json_data')
         process_instance = GeoJSONData(**kwargs)
         return process_instance
 
@@ -541,24 +527,21 @@ class DragonGather(TaskScheduler):
         # input : ids, loop, sta
         uin = 3
         idi = my_random_string(uin)
-        print("====Datawork Gather Iteration===")
-        bprint("idi %s, args %s" % (idi, args))
-        rprint("idi %s, kwargs %s" % (idi, kwargs))
-        await asyncio.sleep(.5)
         loop = asyncio.get_event_loop()
-        log = kwargs.get('log')
         v = int(args[0])
         sta = args[1]
         di, control, (rc, rc_enu), process_data = sta
         sta_init_flag = self.sta_init.get(ids)
         code = self.stations[ids]['code']
         code_db = self.stations[ids]['db']
+        log = kwargs.get('log')
         # self.rethinkdb_origin[ids] = True
         # self.rethinkdb_enu_origin[ids] = True
         """
         Objetos centrales, de operacion
         """
         body, origin, destiny = False, False, False
+        await asyncio.sleep(.5)
 
         if not sta_init_flag:
             # para controlar que?
@@ -579,7 +562,7 @@ class DragonGather(TaskScheduler):
             # rec_enu: destino
             rc = self.add_sta_instance_origin(ids, loop)
             try:
-                conn = await wait_for(rc.async_connect(), timeout=10)
+                await wait_for(rc.async_connect(), timeout=10)
                 rc.set_defaultdb(self.rethinkdb_dbname)
                 await rc.list_dbs()
                 await rc.select_db(self.rethinkdb_dbname)
@@ -589,9 +572,8 @@ class DragonGather(TaskScheduler):
                 origin = True
             except asyncio.TimeoutError as e:
                 idex = my_random_string(uin)
-                log.exception(
-                    "PD_TO1 IDEX %s: Tiempo fuera para conectar db source %s" % rc)
-                kwargs["origin_exception"] = "PD_TO1_00 + %s" % (idex, code)
+                kwargs["origin_exception"] = f"PD_TO1_00 + {idex} + {code} +e"
+                log.exception(kwargs["origin_exception"])
                 self.rethinkdb_origin[ids] = False
                 rc.close()
                 del rc
@@ -620,10 +602,8 @@ class DragonGather(TaskScheduler):
                 destiny = True
             except asyncio.TimeoutError as e:
                 idex = my_random_string(uin)
-                log.exception(
-                    '''PD_TO2 IDEX %s: Tiempo fuera para conectar
-                    db destiny %s''' % (idex, e))
-                kwargs["origin_exception"] = "PD_TO1_00 + %s" % code
+                kwargs["origin_exception"] = f"PD_TO1_00 + {code}"
+                log.exception(kwargs["origin_exception"])
                 self.rethinkdb_origin_enu[ids] = False
                 bprint(kwargs)
                 rc_enu.close()
@@ -647,21 +627,31 @@ class DragonGather(TaskScheduler):
                                             [di, df],
                                             filter_opt,
                                             key)
-                cursor = await shield(wait_for(future, timeout=30))
-            except asyncio.TimeoutError as e:
+                cursor = await wait_for(future, timeout=60)
+            except (socket.error,
+                    ConnectionResetError,
+                    ConnectionAbortedError) as conn_error:
                 idex = my_random_string(uin)
-                log.exception(
-                    '''PD_TO3 IDEX %s: Tiempo fuera para conectar db % s,
-                    al leer datos desde db, tabla %s''' % (
-                        idex, e, table_name))
-                kwargs["origin_exception"] = "PD_TO3_00 + %s" % code
+                kwargs["origin_exception"] = f'''PD_TO3_-1 + {idex} + {code} +
+                {conn_error}, ConnectionClosed rc connection....'''
+                log.exception(kwargs["origin_exception"])
+                self.rethinkdb_origin[ids] = False
+                bprint(kwargs)
+                rc.close()
+                del rc
+                rc = None
+                wargs = [ipt, ids, v, [
+                    di, control, (rc, rc_enu), process_data]]
+                return wargs, kwargs
+            except asyncio.TimeoutError as e:
+                idex = my_random_string(uin)
+                kwargs["origin_exception"] = f"PD_TO3_00 + {idex} + {code} + {e}"
+                log.exception(kwargs["origin_exception"])
                 self.rethinkdb_origin[ids] = False
                 bprint(kwargs)
                 rc.close()
                 del rc
                 rc = None
-                log.exception("PD_TO3 IDEX %s: next iteration" %
-                              (idex, table_name))
                 wargs = [ipt, ids, v, [
                     di, control, (rc, rc_enu), process_data]]
                 return wargs, kwargs
@@ -670,48 +660,70 @@ class DragonGather(TaskScheduler):
                 print("IDEX %s Exception" % idex, ex)
                 print("Error en obtención de data desde rethinkdb")
                 rc.logerror(
-                    "IDEX = %s, Error en la obtención de datos para estación %s en %s" % (
-                        idex, code, di))
-                kwargs["origin_exception"] = "PD_TO4_00 + %s" % code
-                self.rethinkdb_origin[ids] = False
+                    f'''IDEX = {idex}, Error en la obtención de datos para estación {code} en {ex}'''
+                kwargs["origin_exception"]=f"PD_TO4_00 + {code} + {idex} + {ex}"
+                log.exception(kwargs["origin_exception"])
+                self.rethinkdb_origin[ids]=False
                 rc.close()
                 del rc
-                rc = None
-                wargs = [ipt, ids, v, [
+                rc=None
+                wargs=[ipt, ids, v, [
                     di, control, (rc, rc_enu), process_data]]
                 return wargs, kwargs
 
             # signal.message({"msg":"cursor len %s" %len(cursor)})
-            list_ok = []
+            list_ok=[]
             for c in cursor:
                 if check_gsof(c):
                     try:
-                        result = process_data.manage_data(c)
-                        to_plot = geojson2json(result)
-                        to_db = geojson2json(result, destiny='db')
-                        result_enu = await rc_enu.save_data(table_name, to_db)
+                        result=process_data.manage_data(c)
+                        # to_plot = geojson2json(result)
+                        to_db=geojson2json(result, destiny='db')
+                        # result_enu =
+                        try:
+                            await wait_for(
+                                rc_enu.save_data(table_name, to_db),
+                                timeout=60)
+                       except (socket.error,
+                               ConnectionResetError,
+                               ConnectionAbortedError) as conn_error:
+                           idex=my_random_string(uin)
+                           kwargs["origin_exception"]=f'''PD_TO3_-1 + {idex} +
+            {code} + {conn_error}, ConnectionClosed rc_enu connection....'''
+                           log.exception(kwargs["origin_exception"])
+                           self.rethinkdb_origin[ids]=False
+                           bprint(kwargs)
+                           rc.close()
+                           del rc
+                           rc=None
+                           wargs=[ipt, ids, v, [
+                               di, control, (rc, rc_enu), process_data]]
+                           return wargs, kwargsPp
+                        except Exception as e:
+                            bprint(f"Algo pasa con rc_enu....{e}")
                         # send normal json
-                        send = {
+                        msg_result=qjson(result)
+                        send={
                             'channel': 'earlybird',
                             'command': 'add_data',
-                            'data': json.loads(json.dumps(result))}
+                            'data': msg_result}
                         self.dbus_queue.put(send)
                         # send to earlybird
                         await asyncio.sleep(.1)
-                        send_eb = {
+                        send_eb={
                             'channel': 'earlybird',
                             'command': 'add_data',
-                            'data': json.loads(json.dumps(result))}
+                            'data': msg_result}
                         try:
                             self.dbus_queue.put(send_eb)
                         except Exception as e:
                             rprint("Error al enviar send_ev")
                             bprint(e)
-                        intersection = self.status_keys.intersection(c.keys())
+                        intersection=self.status_keys.intersection(c.keys())
                         if intersection:
-                            data_msg = {key: qjson(c.get(key))
+                            data_msg={key: qjson(c.get(key))
                                         for key in intersection}
-                            send = {
+                            send={
                                 'channel': 'status',
                                 'command': 'add_status',
                                 'station': code,
@@ -720,25 +732,24 @@ class DragonGather(TaskScheduler):
                         if self.dbus_geojson_queue:
                             self.dbus_geojson_queue.put(send)
                     except asyncio.TimeoutError as e:
-                        log.exception(
-                            f"PD_TO3: Tiempo fuera para conectar db {e}, al" +
-                            "leer datos desde db")
-                        kwargs["origin_exception"] = "PD_TO5_01 + %s" % code
-                        self.rethinkdb_origin_enu[ids] = False
+                        kwargs["origin_exception"]=f"PD_TO5_01 + {code} + {e}"
+                        log.exception(kwargs["origin_exception"])
+                        self.rethinkdb_origin_enu[ids]=False
                         rc_enu.close()
                         del rc_enu
-                        rc_enu = None
-                        wargs = [ipt, ids, v, [
+                        rc_enu=None
+                        wargs=[ipt, ids, v, [
                             di, control, (rc, rc_enu), process_data]]
                         return wargs, kwargs
                     except Exception as ex:
                         print("WS Error al enviar %s a cola %s" %
                               (code, ex), flush=True)
-                        kwargs["origin_exception"] = "PD_TO5_02 + %s" % code
+                        kwargs["origin_exception"]="PD_TO5_02 + %s" % code
+                        log.exception(kwargs["origin_exception"])
                         bprint(kwargs)
                         rc_enu.close()
                         del rc_enu
-                        rc_enu = None
+                        rc_enu=None
                         return [ipt, ids, v,
                                 [di, control,
                                  (rc, rc_enu),
@@ -746,12 +757,12 @@ class DragonGather(TaskScheduler):
 
                     list_ok.append(c)
                 else:
-                    msg = "Error en estacixon %s  ----> %s" % (code, c)
+                    msg="Error en estacixon %s  ----> %s" % (code, c)
                     await rc.msg_log(msg, "DEBUG")
             if cursor:
-                dt_recv = cursor[-1].get('DT_RECV')
-                di = cursor[-1].get('DT_GEN', dt_recv)
-                control = True
+                dt_recv=cursor[-1].get('DT_RECV')
+                di=cursor[-1].get('DT_GEN', dt_recv)
+                control=True
             return [
                 ipt, ids, v,
                 [di, control, (rc, rc_enu), process_data]
@@ -765,15 +776,10 @@ class DragonGather(TaskScheduler):
         """
         This definition is for collector instance
         """
-        log = FileLog("Gather@Datawork", "CORE_%s" %
+        rprint(f"Init log..... for ipt {ipt}")
+        log=print
+        log=LogFile("Engine@Datawork", "CORE_%s" %
                       ipt, "localhost@pineiden",
-                      path=self.log_path/"engine")
-        self.log_manager[ipt] = log
-        bprint(f"Init filelog on IPT {ipt}")
-        rprint(log)
+                      path=str(self.log_path/"engine"))
+        self.log_manager[ipt]=log
         return [ipt, 1, (None, None, (None, None), None)], {"log": log}
-
-    def manage_tasks(self, ipt):
-        bprint(f"Init task process IPT {ipt}")
-        super().manage_task(ipt)
-        rprint(f"End manage tstk {ipt}")
